%This script implements the methods described in e.g "Congestion Management in Open Access Based on Relative Electrical Distance Using Voltage Stability Criteria"
%Also does the Sobhy stuff in "Network Usage Determination Using a Transformer Analogy"

clear;

load('RandOutputBlock')
load('RandVoltBlock')

define_constants;

BaseSystem = loadcase('case30EHN'); %take it in all ready to go

newplot_mpc(BaseSystem)

export_fig (['case30EHN.png'], '-m4', '-nocrop')



[SortedSystem, SortedYbus, SortingIndex, YBranch] = MakeBusTypeSortedSystem(BaseSystem);

if(isfield(SortedSystem, 'order'))
SortedSystem = rmfield(SortedSystem,'order'); %so it doesn't haunt us
end   
%Need to split the matrices into load and PV parts

NumGenNodes = numel(find(SortedSystem.bus(:, BUS_TYPE) > 1)); %Number of actual generator nodes - doesn't include negative loads

NumLoadNodes = size(SortedSystem.bus,1) - NumGenNodes;

[Ybuspu, Yf, Yt] = makeYbus(SortedSystem);

Ybuspu = full(Ybuspu);
Zbuspu = pinv(Ybuspu);

Ygg = Ybuspu(1:NumGenNodes,1:NumGenNodes);
Ygl = Ybuspu(1:NumGenNodes,NumGenNodes+1:NumGenNodes +NumLoadNodes);
Ylg = Ybuspu(NumGenNodes+1:NumGenNodes + NumLoadNodes,1:NumGenNodes);
Yll = Ybuspu(NumGenNodes+1:NumGenNodes + NumLoadNodes,NumGenNodes+1:NumGenNodes + NumLoadNodes);

Zll = pinv(Yll);
Zgg = pinv(Ygg);

Flg = -Zll * Ylg; %This Flg matrix is a bit magic - it tells us how load voltages depends on each PV voltage. Each row sums to one

Yggm = Ygg -(Ygl * Zll * Ylg); %per formuala given after eqn (6)

LossRec = [];

Vrec = []

SuccessRec = [];

for ThisRun = 1:100

    yalmip('clear')
    
SortedSystem.gen(:, PG) = GenRandOutputs(:, ThisRun)

SortedSystem.gen(:, VG) = 1


SortedSystem = runpf(SortedSystem)
TotalInitialLosses = sum(get_losses(SortedSystem))

TotalInitialExport = SortedSystem.gen(1, PG)

TotalNetWindPower = sum(SortedSystem.gen(2:end, PG)) - sum(SortedSystem.bus(:, PD))

%Needs to be in rectangular complex form!
Vangrad = SortedSystem.bus(:, VA) * pi / 180;

[X, Y] = pol2cart(Vangrad,SortedSystem.bus(:, VM));

VSortedpuRect = X + (Y * i); %like you'd get from a PMU

%split voltage vectors
VgenSortedpuRect = VSortedpuRect(1:NumGenNodes,:);
VloadSortedpuRect = VSortedpuRect(NumGenNodes+1:NumGenNodes + NumLoadNodes,:);

CalcCircPower =  VgenSortedpuRect  .* conj(Yggm  * VgenSortedpuRect);

Voltages = sdpvar(size(VgenSortedpuRect,1),1,'full','complex'); 
CircPower = sdpvar(size(VgenSortedpuRect,1),1,'full','complex'); 

%Constraints = [CircPower == conj(conj(Voltages) .* (Yggm  * Voltages)), ...
% real(CircPower(2:end)) == real(CalcCircPower(2:end)), imag(Voltages(1)) == 0, 0 <= real(Voltages) <= 2, -2 <= imag(Voltages) <= 2]; %NB CHECK CONJUGATES



Constraints = [CircPower == Voltages .* conj(Yggm  * Voltages), ...
 real(CircPower(2:end)) == real(CalcCircPower(2:end)), imag(Voltages(1)) == 0, 0 <= real(Voltages) <= 2, -2 <= imag(Voltages) <= 2]; %NB CHECK CONJUGATES


 %Voltages(1) == SlackMag, 0.9 <= SlackMag <= 1.1 ];

    for ThisVolt = 1:size(Voltages,1)

    Constraints = [Constraints, 0.95 <= sqrtm(real(Voltages(ThisVolt))^2 + imag(Voltages(ThisVolt))^2) <= 1.05];

    end

    for ThisVolt = 2:size(Voltages,1)

 %   Constraints = [Constraints, sqrtm(real(Voltages(ThisVolt))^2 + imag(Voltages(ThisVolt))^2) == 1]; %Bind up everything except the slack

    end

    
      % Constraints = [Constraints, Voltages(4:end) == VgenSortedpuRect(4:end)]; %Only allow slack bus control

    
     OptSettings = sdpsettings('solver','ipopt','verbose',2,'cachesolvers', 1);

  sol = optimize(Constraints, real(CircPower(1)), OptSettings) 

  NovelMaxedExport = real(value(CircPower(1)))*100;
  
abs(value(Voltages))

CopySys = SortedSystem;

CopySys.bus(:, VMIN) = 0.95;
CopySys.bus(:, VMAX) = 1.05;


CopySys.gen(1, PMAX) = 9999
CopySys.gen(1, PMIN) = -9999

CopySys.gen(2:end, PMAX) = CopySys.gen(2:end, PG)+0.001
CopySys.gen(2:end, PMIN) = CopySys.gen(2:end, PG)-0.001 %Don't allow redispatch of P

CopySys.branch(:, RATE_A) = 9999

tic
    results = runopf(CopySys, mpoption('model', 'ac', 'verbose', 3));
    
    TotalACOPFExport = results.gen(1, PG)

 toc
 
 SortedSystem.gen(:, VG) = abs(value(Voltages))
 
 LossSystem = runpf(SortedSystem)

 ValidatedNovelMaxedExport = LossSystem.gen(1, PG)
 
%scatter(real(CircPower)*100, SortedSystem.gen(:, PG))
%scatter(imag(CircPower)*100, SortedSystem.gen(:, QG))
LossRec = [LossRec; [TotalNetWindPower, -TotalInitialExport,-NovelMaxedExport, -ValidatedNovelMaxedExport, -TotalACOPFExport]];
    
Vrec = [Vrec, LossSystem.bus(:, VM), results.bus(:, VM)]

SuccessRec =[SuccessRec; [~sol.problem, LossSystem.success, results.success ]]

end

save('OptRecords.mat', 'SuccessRec', 'Vrec', 'LossRec')

   set(0, 'DefaultAxesFontName', 'Times');
   set(0,'DefaultAxesFontSize',18);
   set(0,'DefaultFigureColor','w')
    set(0,'defaulttextinterpreter','tex')


 max((LossRec(:,4) ./ LossRec(:,2) - 1) * 100)
 min((LossRec(:,4) ./ LossRec(:,2) - 1) * 100)
 
 
 max((LossRec(:,4) ./ LossRec(:,5) - 1) * 100)
 min((LossRec(:,4) ./ LossRec(:,5) - 1) * 100)
 
 
 green = [44,162,95]/255
 
 
lbig = linspace(0.5, 1.5, 11)

figure

%h = histogram(LossRec(:,1) - LossRec(:,4),  l, 'EdgeColor', 'w', 'FaceColor', green)

h = histogram( (LossRec(:,4) ./ LossRec(:,2) - 1) * 100, lbig, 'EdgeColor', 'w', 'FaceColor', green)

axis([0.5 1.5 0 40])

xlabel('Increase in export (%) \rightarrow')
ylabel('Periods \rightarrow')

box off

set(gca, 'Xtick', [0.5, 1, 1.5])
set(gca, 'Ytick', [0, 20, 40])

set(gca, 'YtickLabels', {'', '20', '40'})
set(gca, 'XtickLabels', {'0.5%','1%','1.5%'})


FigWidth = 7 * 2
FigHeight = 3  * 2

%# set size of figure's "drawing" area on screen
set(gcf, 'Units','centimeters', 'Position',[0 0 FigWidth FigHeight])
%# create some plot, and make axis fill entire figure
set(gca, 'Units','normalized')
set(gca, 'Position',[0.16 0.29 0.78 0.67])


 export_fig('HistogramFirstComparePerc', '-m4', '-nocrop')

 
 lsmall = linspace(0, 0.25, 11)

 
figure
h = histogram( (LossRec(:,4) ./ LossRec(:,5) - 1) * 100, lsmall,  'EdgeColor', 'w', 'FaceColor', green)


xlabel('Increase in export (%) \rightarrow')
ylabel('Periods \rightarrow')

box off

axis([0 0.25 0 40])

set(gca, 'Xtick', [0, 0.125, 0.25])
set(gca, 'Ytick', [0, 20, 40])

set(gca, 'YtickLabels', {'', '20', '40'})
set(gca, 'XtickLabels', {'0%', '0.125%', '0.25%'})


%# set size of figure's "drawing" area on screen
set(gcf, 'Units','centimeters', 'Position',[0 0 FigWidth FigHeight])
%# create some plot, and make axis fill entire figure
set(gca, 'Units','normalized')
set(gca, 'Position',[0.16 0.29 0.78 0.67])

export_fig('HistogramSecondComparePerc', '-m4', '-nocrop')











































