%This script implements the methods described in e.g "Congestion Management in Open Access Based on Relative Electrical Distance Using Voltage Stability Criteria"
%Also does the Sobhy stuff in "Network Usage Determination Using a Transformer Analogy"

clear;


 GenRandOutputs = raylrnd(7, 13, 100)

% GenRandOutputs = wblrnd(8.29,2.93,13,100) %Parameters from S. N. Chowdhury and S. Dhawan, "Statistical estimation for fitting wind speed distribution," 2016 International Conference on Energy Efficient Technologies for Sustainability (ICEETS), Nagercoil, 2016, pp. 34-37.     

GenRandOutputs = fcnpowercurve(GenRandOutputs,8)

mean(GenRandOutputs(:))

hist(GenRandOutputs(1,:))

%save('PostReviewGenRandOutputs.mat', 'GenRandOutputs')
load('PostReviewGenRandOutputs')


%load('RandVoltBlock')

define_constants;

BaseSystem = loadcase('case30EHN'); %take it in all ready to go

newplot_mpc(BaseSystem)

export_fig (['case30EHN.png'], '-m4', '-nocrop')


[SortedSystem, SortedYbus, SortingIndex, YBranch] = MakeBusTypeSortedSystem(BaseSystem);

if(isfield(SortedSystem, 'order'))
SortedSystem = rmfield(SortedSystem,'order'); %so it doesn't haunt us
end   
%Need to split the matrices into load and PV parts

NumGenNodes = numel(find(SortedSystem.bus(:, BUS_TYPE) > 1)); %Number of actual generator nodes - doesn't include negative loads

NumLoadNodes = size(SortedSystem.bus,1) - NumGenNodes;

[Ybuspu, Yf, Yt] = makeYbus(SortedSystem);

Ybuspu = full(Ybuspu);
Zbuspu = pinv(Ybuspu);

Ygg = Ybuspu(1:NumGenNodes,1:NumGenNodes);
Ygl = Ybuspu(1:NumGenNodes,NumGenNodes+1:NumGenNodes +NumLoadNodes);
Ylg = Ybuspu(NumGenNodes+1:NumGenNodes + NumLoadNodes,1:NumGenNodes);
Yll = Ybuspu(NumGenNodes+1:NumGenNodes + NumLoadNodes,NumGenNodes+1:NumGenNodes + NumLoadNodes);

Zll = pinv(Yll);
Zgg = pinv(Ygg);

Flg = -Zll * Ylg; %This Flg matrix is a bit magic - it tells us how load voltages depends on each PV voltage. Each row sums to one

Yggm = Ygg -(Ygl * Zll * Ylg); %per formuala given after eqn (6)

LossRec = [];

Vrec = []

SuccessRec = [];

NovelTimeRec = [];
ACOPFTimeRec = [];



for ThisRun = 1:100

   % yalmip('clear')
    
   
   
SortedSystem.gen(:, PG) = GenRandOutputs(:, ThisRun)

SortedSystem.gen(:, VG) = 1

SortedSystem = runpf(SortedSystem)

TotalInitialLosses = sum(get_losses(SortedSystem))

TotalInitialExport = SortedSystem.gen(1, PG)

TotalNetWindPower = sum(SortedSystem.gen(2:end, PG)) - sum(SortedSystem.bus(:, PD))

%Needs to be in rectangular complex form!
Vangrad = SortedSystem.bus(:, VA) * pi / 180;

[X, Y] = pol2cart(Vangrad,SortedSystem.bus(:, VM));

VSortedpuRect = X + (Y * i); %like you'd get from a PMU

%split voltage vectors
VgenSortedpuRect = VSortedpuRect(1:NumGenNodes,:);
VloadSortedpuRect = VSortedpuRect(NumGenNodes+1:NumGenNodes + NumLoadNodes,:);

CalcCircPower =  VgenSortedpuRect  .* conj(Yggm  * VgenSortedpuRect);

Voltages = sdpvar(size(VgenSortedpuRect,1),1,'full','complex'); 
CircPower = sdpvar(size(VgenSortedpuRect,1),1,'full','complex'); 

%Constraints = [CircPower == conj(conj(Voltages) .* (Yggm  * Voltages)), ...
% real(CircPower(2:end)) == real(CalcCircPower(2:end)), imag(Voltages(1)) == 0, 0 <= real(Voltages) <= 2, -2 <= imag(Voltages) <= 2]; %NB CHECK CONJUGATES



Constraints = [CircPower == Voltages .* conj(Yggm  * Voltages), ...
 real(CircPower(2:end)) == real(CalcCircPower(2:end)), imag(Voltages(1)) == 0, 0 <= real(Voltages) <= 2, -2 <= imag(Voltages) <= 2]; %NB CHECK CONJUGATES


 %Voltages(1) == SlackMag, 0.9 <= SlackMag <= 1.1 ];

    for ThisVolt = 1:size(Voltages,1)

    Constraints = [Constraints, 0.95 <= sqrtm(real(Voltages(ThisVolt))^2 + imag(Voltages(ThisVolt))^2) <= 1.05];

    end

    for ThisVolt = 2:size(Voltages,1)

 %   Constraints = [Constraints, sqrtm(real(Voltages(ThisVolt))^2 + imag(Voltages(ThisVolt))^2) == 1]; %Bind up everything except the slack

    end

    
      % Constraints = [Constraints, Voltages(4:end) == VgenSortedpuRect(4:end)]; %Only allow slack bus control

    
     OptSettings = sdpsettings('solver','ipopt','verbose',0,'cachesolvers', 1);

tic
  sol = optimize(Constraints, real(CircPower(1)), OptSettings)  %NB need a time record!
NovelTime = toc;
  NovelTimeRec = [NovelTimeRec; NovelTime];

  
  NovelMaxedExport = real(value(CircPower(1)))*100;
  
abs(value(Voltages))

CopySys = SortedSystem;

CopySys.bus(:, VMIN) = 0.95;
CopySys.bus(:, VMAX) = 1.05;


CopySys.gen(1, PG) = 0 %So we don't inherit defaults

CopySys.gen(1, PMAX) = 9999
CopySys.gen(1, PMIN) = -9999

CopySys.gen(2:end, PMAX) = CopySys.gen(2:end, PG)+0.0001
CopySys.gen(2:end, PMIN) = CopySys.gen(2:end, PG)-0.0001 %Don't allow redispatch of P

CopySys.branch(:, RATE_A) = 9999

tic
    results = runopf(CopySys, mpoption('model', 'ac', 'verbose', 3))
ACOPFtime = toc
ACOPFTimeRec = [ACOPFTimeRec; ACOPFtime];

    TotalACOPFExport = results.gen(1, PG)


 SortedSystem.gen(:, VG) = abs(value(Voltages))
 
 LossSystem = runpf(SortedSystem)

 ValidatedNovelMaxedExport = LossSystem.gen(1, PG)
 
%scatter(real(CircPower)*100, SortedSystem.gen(:, PG))
%scatter(imag(CircPower)*100, SortedSystem.gen(:, QG))
LossRec = [LossRec; [TotalNetWindPower, -TotalInitialExport,-NovelMaxedExport, -ValidatedNovelMaxedExport, -TotalACOPFExport]];
    
Vrec = [Vrec, LossSystem.bus(:, VM), results.bus(:, VM)]

SuccessRec =[SuccessRec; [~sol.problem, LossSystem.success, results.success ]]

end

DudACOPFruns = find(SuccessRec(:, 3) == 0)
GoodACOPFruns = find(SuccessRec(:, 3) == 1)

LossRec(DudACOPFruns, 5) = NaN

TotalBaseExports = round(sum(LossRec(GoodACOPFruns, 2)))
TotalNovelExports = round(sum(LossRec(GoodACOPFruns, 4)))
TotalACOPFlExports = round(sum(LossRec(GoodACOPFruns, 5)))

max((LossRec(GoodACOPFruns, 1)))


save('PostReviewOptRecords.mat', 'SuccessRec', 'Vrec', 'LossRec')

   set(0, 'DefaultAxesFontName', 'Times');
   set(0,'DefaultAxesFontSize',16);
   set(0,'DefaultFigureColor','w')
    set(0,'defaulttextinterpreter','tex')


 max((LossRec(:,4) ./ LossRec(:,2) - 1) * 100)
 min((LossRec(:,4) ./ LossRec(:,2) - 1) * 100)
 
 
 max((LossRec(:,4) ./ LossRec(:,5) - 1) * 100)
 min((LossRec(:,4) ./ LossRec(:,5) - 1) * 100)
 
 
 green = [44,162,95]/255
 
 
lbig = linspace(0, 2, 9)

figure

%h = histogram((LossRec(:,4) ./ LossRec(:,2) - 1) * 100, 'EdgeColor', 'w', 'FaceColor', green)
%max((LossRec(:,4) ./ LossRec(:,2) - 1) * 100)

h = histogram((LossRec(:,4) ./ LossRec(:,2) - 1) * 100, lbig, 'EdgeColor', 'w', 'FaceColor', green)

axis([0 2 -0.3 40])

xlabel('Increase in total export (%) \rightarrow')
ylabel('Periods \rightarrow')


set(gca, 'Xtick', [lbig([1,5,9])])
set(gca, 'Ytick', [0, 20, 40])

set(gca, 'YtickLabels', {'', '20', '40'})
set(gca, 'XtickLabels', {'0%','1%','2%'})


FigWidth = 7 * 2
FigHeight = 3  * 2 * 0.8

box off

%# set size of figure's "drawing" area on screen
set(gcf, 'Units','centimeters', 'Position',[0 0 FigWidth FigHeight])
%# create some plot, and make axis fill entire figure
set(gca, 'Units','normalized')
set(gca, 'Position',[0.12 0.35 0.82 0.58])


 export_fig('PostReviewHistogramFirstComparePerc2', '-m4', '-nocrop')

 

 max( (LossRec(:,4) ./ LossRec(:,5) - 1) * 100)
 
 h = histogram( (LossRec(:,4) ./ LossRec(:,5) - 1) * 100,  'EdgeColor', 'w', 'FaceColor', green)

 
  lsmall = linspace(0, 0.2, 9)
figure
h = histogram( (LossRec(:,4) ./ LossRec(:,5) - 1) * 100, lsmall,  'EdgeColor', 'w', 'FaceColor', green)


xlabel('Increase in total export (%) \rightarrow')
ylabel('Periods \rightarrow')

box off

axis([-0.0005 0.2 -0.3 40])

set(gca, 'Xtick', [lsmall([1,5,9])])
set(gca, 'Ytick', [0, 20, 40])

set(gca, 'YtickLabels', {'', '20', '40'})
set(gca, 'XtickLabels', {'0%', '0.1%', '0.2%'})


%# set size of figure's "drawing" area on screen
set(gcf, 'Units','centimeters', 'Position',[0 0 FigWidth FigHeight])
%# create some plot, and make axis fill entire figure
set(gca, 'Units','normalized')
set(gca, 'Position',[0.12 0.35 0.82 0.58])

export_fig('PostReviewHistogramSecondComparePerc2', '-m4', '-nocrop')


mean(ACOPFTimeRec)

mean(NovelTimeRec)








































